package com.hungraim.ltc.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.assessment.*;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvDisableLevel;
import com.hungraim.ltc.pojo.vo.assessment.*;
import com.hungraim.ltc.service.TaskService;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.SerialNumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @author czz
 */
@Service
public class TaskServiceImpl implements TaskService {

    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private AprEmpMapper aprEmpMapper;
    @Autowired
    private AprReportMapper aprReportMapper;
    @Autowired
    private TaskEmpMapper taskEmpMapper;
    @Autowired
    private AprPersonInfoMapper personInfoMapper;
    @Autowired
    private TaskPushMapper taskPushMapper;
    @Autowired
    private CfgSrvDisableLevelMapper cfgSrvDisableLevelMapper;

    @Override
    public IPage<TaskDistributionVO> taskDistributionList(String organId, String provincial, String city,
                                                          String street, String realName, String certiCode, String taskName, String taskTel,
                                                          String daysRemaining, String status, String isReturn, String area,
                                                          int pageNum, int pageSize) {
        List<String> organIds = null;
        if (StrUtil.isNotBlank(organId)) {
            organIds = Arrays.asList(organId.split(","));
        }
        return taskMapper.taskDistributionList(new Page<>(pageNum, pageSize), organIds, provincial, city,
                street, realName, certiCode, taskName, taskTel, daysRemaining, status, isReturn, area);
    }

    @Override
    public List<TaskReportVO> taskReportList(String name, String certiCode, String tel) {
        return taskMapper.taskReportList(name, certiCode, tel);
    }

    @Override
    public List<AprEmpVO> aprEmpList(String organName, String name, String tel, String groupLeader) {
        return taskMapper.aprEmpList(organName, name, tel, groupLeader);
    }

    @Override
    public void addReport(Integer taskId, Integer person1, Integer person2, Integer person3, Date evaluateTime, Integer disableLevelId, String polNo, String remark) {
        ChAprReport report = new ChAprReport();
        report.setTaskId(taskId);
        report.setEffTime(new Date());
        report.setRemark(remark);
        report.setDisableLevelId(disableLevelId);
        report.setPolNo(polNo);
        report.setStatus(0);
        report.setEvaluateTime(evaluateTime);
        aprReportMapper.insert(report);
        //person1
        ChAprPersonInfo person1Info = new ChAprPersonInfo();
        person1Info.setReportId(report.getReportId());
        person1Info.setEmpId(person1);
        person1Info.setFcd(new Date());
        personInfoMapper.insert(person1Info);
        if (person2 != null) {
            ChAprPersonInfo personInfo = new ChAprPersonInfo();
            personInfo.setReportId(report.getReportId());
            personInfo.setEmpId(person1);
            personInfo.setFcd(new Date());
            personInfoMapper.insert(personInfo);
        }
        if (person3 != null) {
            ChAprPersonInfo personInfo = new ChAprPersonInfo();
            personInfo.setReportId(report.getReportId());
            personInfo.setEmpId(person1);
            personInfo.setFcd(new Date());
            personInfoMapper.insert(personInfo);
        }
    }

    @Override
    public void updateReport(Integer reportId, Integer person1, Integer person2, Integer person3, Date evaluateDate, Integer disableLevelId, String polNo, String remark) {
        //update鉴定报告
        ChAprReport chAprReport = aprReportMapper.selectById(reportId);
        chAprReport.setDisableLevelId(disableLevelId);
        chAprReport.setPolNo(polNo);
        chAprReport.setRemark(remark);
        chAprReport.setEvaluateTime(evaluateDate);
        aprReportMapper.updateById(chAprReport);
        //修改鉴定报告人员(有就update,没有就insert)
        List<ChAprPersonInfo> personInfos = personInfoMapper.selectList(new QueryWrapper<ChAprPersonInfo>().eq("report_id", reportId));
        if (person1 != null) {
            if (personInfos.size() >= 1) {
                ChAprPersonInfo personInfo = personInfos.get(0);
                personInfo.setEmpId(person1);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person1);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
        if (person2 != null) {
            if (personInfos.size() >= 2) {
                ChAprPersonInfo personInfo = personInfos.get(0);
                personInfo.setEmpId(person2);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person2);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
        if (person3 != null) {
            if (personInfos.size() >= 3) {
                ChAprPersonInfo personInfo = personInfos.get(0);
                personInfo.setEmpId(person3);
                personInfoMapper.updateById(personInfo);
            } else {
                ChAprPersonInfo personInfo = new ChAprPersonInfo();
                personInfo.setReportId(chAprReport.getReportId());
                personInfo.setEmpId(person3);
                personInfo.setFcd(new Date());
                personInfoMapper.insert(personInfo);
            }
        }
    }

    @Override
    public Result<Object> allotTask(String taskId, Integer empId) {
        ChAprEmp chAprEmp = aprEmpMapper.selectById(empId);
        if (chAprEmp == null || chAprEmp.getGroupLeader() == null || chAprEmp.getGroupLeader() != 0) {
            //不是组长
            return Result.failed("评定人员必须是组长!");
        }
        String[] taskIds = taskId.split(",");
        Date assignTime = new Date();//任务分配日期
        for (String id : taskIds) {
            ChAprTask task = taskMapper.selectById(id);
            if (task == null) {
                return Result.failed("无效的taskId!");
            }
            task.setAprEmpId(empId);
            task.setAssignTime(assignTime);
            int count = taskMapper.updateById(task);
            if (count != 1) {
                return Result.failed("修改失败!");
            }
            //生成组
            ChAprTaskPush taskPush = new ChAprTaskPush();
            taskPush.setTime(new Date());
            taskPush.setGroupNumber(SerialNumberUtils.getSerialNumber(empId.toString()));
            taskPush.setTaskId(Long.getLong(id));
            taskPush.setEmpId(empId.longValue());
            taskPush.setStatus((short) 1);
            taskPush.setIsManual((short) 1);
            taskPushMapper.insert(taskPush);
        }
        return Result.success();
    }

    @Override
    public List<TaskGroupVO> taskGroupList(String name, String certiCode, String tel) {
        return taskMapper.taskGroupList(name, certiCode, tel);
    }

    @Override
    public Result<Object> taskGroup(Integer pushId, Integer status) {
        ChAprTaskPush chAprTaskPush = taskPushMapper.selectById(pushId);
        if (chAprTaskPush == null) {
            return Result.failed("不存在的分组!");
        }
        chAprTaskPush.setStatus(status.shortValue());
        taskPushMapper.updateById(chAprTaskPush);
        return Result.success();
    }

    @Override
    public List<ChCfgSrvDisableLevel> cfgSrvDisableLevelList() {
        return cfgSrvDisableLevelMapper.selectList(new QueryWrapper<>());
    }

    @Override
    public List<PersonInfoVO> personInfoList(String reportId) {
        return taskMapper.personInfoList(reportId);
    }
}